use chrono::Local;
use dataworker::dtos::Root;
use dataworker::insert_document;
use sqlx::postgres::PgPoolOptions;

use dotenvy::dotenv;
use reqwest_middleware::ClientBuilder;
use reqwest_retry::{policies::ExponentialBackoff, RetryTransientMiddleware};
use std::env;
use std::io::Write;
use tokio::task;
use tokio::time::{sleep, Duration};

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    dotenv().ok();
    let stations: Vec<_> = env::var("STATIONS")?
        .split(',')
        .map(|s| s.to_string())
        .collect();
    let _token = env::var("VRN_TOKEN")?;
    let req = env::var("REQ")?;
    let url = env::var("DATABASE_URL")?;
    let conn = PgPoolOptions::new()
        .max_connections(8)
        .acquire_timeout(Duration::from_secs(3600))
        .connect(&url)
        .await?;

    let total = stations.len();
    let mut count = 0;

    let client = ClientBuilder::new(reqwest::Client::new())
        .with(RetryTransientMiddleware::new_with_policy(
            ExponentialBackoff::builder().build_with_max_retries(3),
        ))
        .build();

    loop {
        let mut handles = vec![];
        std::io::stdout().flush().unwrap();
        //interval.tick().await;
        let date = Local::now();
        //println!("Ticked on {}", &date.format("%Y.%m.%d %H:%M"));
        for station_batch in &mut stations.chunks(50000) {
            for station in station_batch {
                let station = station.clone();
                let req = req.clone();
                let client = client.clone();
                let mut conn = conn.clone();
                count += 1;
                handles.push(task::spawn(async move {
                    let req = req
                        .replace("TEMPLATE_ID", &station)
                        .replace("TEMPLATE_DATE", &date.format("%Y%m%d").to_string())
                        .replace("TEMPLATE_TIME", &date.format("%H%M").to_string());
                    print!("\x1B[2K\rFetching {}/{} ({}) ", count, total, station);
                    std::io::stdout().flush().unwrap();
                    if let Ok(doc) = client
                        .get(req.replace("TEMPLATE_DEPARR", "arr"))
                        //.header("Authorization", &token)
                        .send()
                        .await
                        .unwrap()
                        .json::<Root>()
                        .await
                    {
                        match insert_document(doc, &mut conn).await {
                            Ok(_) => {}
                            Err(e) => println!("\x1B[2K\rError inserting document: {}", e),
                        }
                    } else {
                        println!("\x1B[2K\rFailed to fetch arrivals for station {}", station);
                    }
                    /*
                    if let Ok(doc) = client
                        .get(req.replace("TEMPLATE_DEPARR", "dep"))
                        //.header("Authorization", &token)
                        .send()
                        .await
                        .unwrap()
                        .json::<Root>()
                        .await
                    {
                        match insert_document(doc, &mut conn).await {
                            Ok(_) => {}
                            Err(e) => println!("\x1B[2K\rError inserting document: {}", e),
                        }
                    } else {
                        println!("\x1B[2K\rFailed to fetch departures for station {}", station);
                    }

                         */
                    //sleep(Duration::from_millis(100)).await;
                }));
                sleep(Duration::from_millis(10)).await;
            }
            for handle in handles {
                handle.await?;
            }
            handles = vec![];

        }
        count = 0;
    }
}
